Package: DatabaseConnector
Type: Package
Title: Connecting to Various Database Platforms
Version: 7.2.0
Date: 2026-06-26
Authors@R: c(
    person("Martijn", "Schuemie", email = "schuemie@ohdsi.org", role = c("aut", "cre")),
    person("Marc", "Suchard", role = c("aut")),
    person("Adam", "Black", role = c("aut")),
    person("Observational Health Data Science and Informatics", role = c("cph")),
    person("Microsoft Inc.", role = c("cph"), comment = "SQL Server JDBC driver"),
    person("PostgreSQL Global Development Group", role = c("cph"), comment = "PostgreSQL JDBC driver"),
    person("Oracle Inc.", role = c("cph"), comment = "Oracle  JDBC driver"),
    person("Amazon Inc.", role = c("cph"), comment = "RedShift  JDBC driver")
  )
Description: An R 'DataBase Interface' ('DBI') compatible interface to various database platforms ('PostgreSQL', 'Oracle', 'Microsoft SQL Server', 
    'Amazon Redshift', 'Microsoft Parallel Database Warehouse', 'IBM Netezza', 'Apache Impala', 'Google BigQuery', 'Snowflake', 'Spark', 'SQLite', 
    and 'InterSystems IRIS'). Also includes support for fetching data as 'Andromeda' objects. Uses either 'Java Database Connectivity' ('JDBC') or 
    other 'DBI' drivers to connect to databases.
SystemRequirements: Java (>= 8)
Depends: R (>= 4.0.0)
Imports: rJava, SqlRender (>= 1.19.2), methods, stringr, readr, rlang,
        utils, DBI (>= 1.0.0), urltools, bit64, checkmate, digest,
        dbplyr (>= 2.6.0)
Suggests: aws.s3, R.utils, withr, testthat, DBItest, knitr, rmarkdown,
        RSQLite, ssh, Andromeda, dplyr, RPostgres, odbc, duckdb,
        bigrquery, pool, ParallelLogger, AzureStor, data.table
License: Apache License
VignetteBuilder: knitr
URL: https://ohdsi.github.io/DatabaseConnector/,
        https://github.com/OHDSI/DatabaseConnector
BugReports: https://github.com/OHDSI/DatabaseConnector/issues
Copyright: See file COPYRIGHTS
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-26 14:15:11 UTC; schuemie
Author: Martijn Schuemie [aut, cre],
  Marc Suchard [aut],
  Adam Black [aut],
  Observational Health Data Science and Informatics [cph],
  Microsoft Inc. [cph] (SQL Server JDBC driver),
  PostgreSQL Global Development Group [cph] (PostgreSQL JDBC driver),
  Oracle Inc. [cph] (Oracle JDBC driver),
  Amazon Inc. [cph] (RedShift JDBC driver)
Maintainer: Martijn Schuemie <schuemie@ohdsi.org>
Repository: CRAN
Date/Publication: 2026-06-26 14:50:02 UTC
