# Generated by roxygen2: do not edit by hand

S3method(compileReconnectCode,DatabaseConnectorDbiConnection)
S3method(compileReconnectCode,default)
S3method(dbplyr_edition,DatabaseConnectorConnection)
S3method(disconnect,DatabaseConnectorDbiConnection)
S3method(disconnect,default)
S3method(getCatalogs,DatabaseConnectorDbiConnection)
S3method(getCatalogs,default)
S3method(getSchemaNames,DatabaseConnectorDbiConnection)
S3method(getSchemaNames,default)
S3method(getServer,DatabaseConnectorDbiConnection)
S3method(getServer,default)
S3method(insertTable,DatabaseConnectorDbiConnection)
S3method(insertTable,DatabaseConnectorJdbcConnection)
S3method(insertTable,default)
S3method(listDatabaseConnectorColumns,DatabaseConnectorDbiConnection)
S3method(listDatabaseConnectorColumns,default)
S3method(sql_escape_logical,DatabaseConnectorJdbcConnection)
S3method(sql_query_select,DatabaseConnectorJdbcConnection)
S3method(sql_translation,DatabaseConnectorDbiConnection)
S3method(sql_translation,DatabaseConnectorJdbcConnection)
export(DatabaseConnectorDriver)
export(assertTempEmulationSchemaSet)
export(computeDataHash)
export(connect)
export(createConnectionDetails)
export(createDbiConnectionDetails)
export(createZipFile)
export(dbms)
export(disconnect)
export(downloadJdbcDrivers)
export(dropEmulatedTempTables)
export(executeSql)
export(existsTable)
export(extractQueryTimes)
export(getAvailableJavaHeapSpace)
export(getTableNames)
export(inDatabaseSchema)
export(insertTable)
export(isSqlReservedWord)
export(querySql)
export(querySqlToAndromeda)
export(renderTranslateExecuteSql)
export(renderTranslateQueryApplyBatched)
export(renderTranslateQuerySql)
export(renderTranslateQuerySqlToAndromeda)
export(requiresTempEmulation)
export(sql_escape_logical)
export(sql_query_select)
exportClasses(DatabaseConnectorDbiResult)
exportClasses(DatabaseConnectorDriver)
exportClasses(DatabaseConnectorJdbcResult)
exportMethods(dbAppendTable)
exportMethods(dbCanConnect)
exportMethods(dbClearResult)
exportMethods(dbColumnInfo)
exportMethods(dbConnect)
exportMethods(dbCreateTable)
exportMethods(dbDisconnect)
exportMethods(dbExecute)
exportMethods(dbExistsTable)
exportMethods(dbFetch)
exportMethods(dbGetInfo)
exportMethods(dbGetQuery)
exportMethods(dbGetRowCount)
exportMethods(dbGetRowsAffected)
exportMethods(dbGetStatement)
exportMethods(dbHasCompleted)
exportMethods(dbIsReadOnly)
exportMethods(dbIsValid)
exportMethods(dbListFields)
exportMethods(dbListTables)
exportMethods(dbReadTable)
exportMethods(dbRemoveTable)
exportMethods(dbSendQuery)
exportMethods(dbSendStatement)
exportMethods(dbUnloadDriver)
exportMethods(dbWriteTable)
import(DBI)
import(methods)
import(rJava)
importFrom(bit64,integer64)
importFrom(dbplyr,dbplyr_edition)
importFrom(dbplyr,sql_escape_logical)
importFrom(dbplyr,sql_query_select)
importFrom(dbplyr,sql_translation)
importFrom(rlang,abort)
importFrom(rlang,inform)
importFrom(rlang,warn)
importFrom(utils,download.file)
importFrom(utils,install.packages)
importFrom(utils,menu)
importFrom(utils,object.size)
importFrom(utils,packageVersion)
importFrom(utils,read.csv)
importFrom(utils,sessionInfo)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
importFrom(utils,unzip)
importFrom(utils,write.csv)
importFrom(utils,write.table)
