% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DBI.R
\name{DatabaseConnectorDriver}
\alias{DatabaseConnectorDriver}
\title{Create a DatabaseConnectorDriver object}
\usage{
DatabaseConnectorDriver()
}
\description{
Create a DatabaseConnectorDriver object
}
