% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DBI.R
\docType{class}
\name{DatabaseConnectorDbiResult-class}
\alias{DatabaseConnectorDbiResult-class}
\title{DatabaseConnector DBI results class.}
\description{
DatabaseConnector DBI results class.
}
\keyword{internal}
